#pragma once
#include "resource.h"
#include "afxcmn.h"

#include "AcDbAssocManager.h"
#include "AcDbAssocNetwork.h"
#include "AcGeomConstraint.h"
#include "AcImplicitConstr.h"
#include "AcDbAssocVariable.h"
#include "AcDbAssocDependency.h"
#include "AcDbAssocPersSubentIdPE.h"
#include "AcDbAssocValueDependency.h"
#include "AcDbAssoc2dConstraintGroup.h" // For AcDbAssoc2dConstraintGroup
#include "AcDbAssocDimDependencyBodyBase.h"
#include "AcDbAssocDimDependencyBody.H"
#include "AcDbAssocGlobal.h"
#include "AcDbAssocGeomDependency.h"
#include "AcConstrainedGeometry.h"
#include "dbeval.h"
#include "afxwin.h"
// CConstrainListDlg dialog

class CConstrainTreeItem
{
public:
	CConstrainTreeItem()
	{
		m_Id = AcDbObjectId::kNull;
		m_bConstraintGroupNode = false;
		m_GroupNodeId = 0;
	}

	~CConstrainTreeItem()
	{
	}

	AcDbObjectId m_Id;
	bool m_bConstraintGroupNode;
	AcConstraintGroupNodeId m_GroupNodeId;
	
};

class CConstrainListDlg : public CAdUiDialog
{
	DECLARE_DYNAMIC(CConstrainListDlg)

public:
	CConstrainListDlg(CWnd* pParent = NULL);   // standard constructor
	
	
	void SetObj( AcDbObject* obj);


	virtual ~CConstrainListDlg();

// Dialog Data
	enum { IDD = IDD_CONSTRAINDBG_DLG };

private:
	//AcDbObjectIdArray    m_treeObjIdList;
	AcArray<CConstrainTreeItem *>m_treeObjectArray;
	AcDbObjectId m_currentSpaceId;

	void InitializeDisplay();


	//helper functions
	void addGeometryDependencyObjects(AcConstrainedGeometry *pGeometry, AcDbAssoc2dConstraintGroup *pGroup, HTREEITEM parent);
	void addDimDependencyDependencyObjects(AcConstrainedGeometry *pGeometry, HTREEITEM parent);

	//list control
	void addSeperatorLine(LPCTSTR className);
	void addToDataList(LPCTSTR field, LPCTSTR value, bool addQuotes = false);
	void drawPropsList(CListCtrl& dataList);
	void buildColumns(CListCtrl& dataList);


	//display
	void display(AcDbObject* obj);
	void display_AcDbAssocAction(AcDbAssocAction * obj);
	void display_AcDbAssocNetwork(AcDbAssocNetwork * obj);
	void display_AcDbAssocVariable(AcDbAssocVariable *obj);


	//display
	void display_AcConstraintGroupNode(AcConstraintGroupNode *pObj);
	void display_AcConstrainedGeometry(AcConstrainedGeometry *pObj);
	void display_AcConstrainedCurve(AcConstrainedCurve *pObj);
	void display_AcConstrainedCircle(AcConstrainedCircle *pObj);
	void display_AcConstrainedArc(AcConstrainedArc* pObj);

protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support

	DECLARE_MESSAGE_MAP()
public:
	CTreeCtrl m_ConstrainTree;
	CListCtrl m_ConstrainList;

    CStringList		m_fieldStrList;
    CStringList		m_valueStrList;

	AcDbObject*     m_obj;



	virtual BOOL OnInitDialog();
	void displayCurrent(int index);
	void AddGeomConstraint(AcDbAssoc2dConstraintGroup *pGroup, HTREEITEM parent);
	HTREEITEM addOneTreeItem(LPCTSTR name, const AcDbObjectId& objId, HTREEITEM parent, bool sort, const bool &bConstraintGroupNode = false, const AcConstraintGroupNodeId &nodeId = 0);

	AcDbObjectId getAssocNetwork(bool createIfNotExist, const AcDbObjectId &spaceId, AcDbDatabase* pDatabase);
	void AddAssocAction(const AcDbObjectId &networkId, HTREEITEM parent);
	afx_msg void OnTvnSelchangedConstrainTree(NMHDR *pNMHDR, LRESULT *pResult);

	afx_msg void OnCbnSelchangeComboSpace();
	CComboBox m_SpaceComboBox;
};
